unit ugter_combinado_horario;

interface


{xDEFINE dbgc}
uses
  SysUtils, Classes, xmatdefs, uGTer, unodos,
  Math,
  uglobs,
  umipsimplex,
  usimplex,
  ufichasLPD,
  ufechas,
  ucosa, uCosaConNombre,
  ugter_combinado,
  uEstados,
  uconstantesSimSEE, uFuentesAleatorias;

resourcestring
  rsGeneradorTermicoCombinadoHorario = 'Generador térmico combinado horario';

type

  { TFichaGTer_combinado_horario }

  TFichaGTer_combinado_horario = class(TFichaGTer_combinado)

  public

    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    horasPurga: NReal;
    horasTVPleno: NReal;
    NDisc: integer;
    costoComb: NReal;
    disp_cald: Nreal;
    tRepHoras_cald: NReal;

    (**************************************************************************)

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PMin_TG, PMax_TG: NReal; cv_min_TG, cv_TG, cv_NoCombustible_TG: NReal;
      disp_TG: NReal; tRepHoras_TG: NReal; PMin_TV, PMax_TV: NReal;
      cv_min_TV, cv_TV, cv_NoCombustible_TV: NReal; disp_TV: NReal;
      tRepHoras_TV: NReal; factorPotenciaTVdivTG: NReal;
      indicePreciosPorCombustible: TFuenteAleatoria; bornePreciosPorCombustible: string;
      HayRestriccionEmaxPasoDeTiempo: boolean; EmaxPasoDeTiempo: NReal;
      PagoPorDisponibilidad_USD_MWh, PagoPorEnergia_USD_MWh: NReal;
      horasPurga: NReal; horasTVPleno: NReal; NDisc: integer; costoComb: NReal;
      disp_cald: NReal; tRepHoras_cald: NReal);

    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(f: TArchiTexto); override;

    procedure generarLineaResumen(var archi: TextFile); override;
    function infoAd_: string; override;
    destructor destroy; override;


  published

  end;


  { TGTer_combinado_horario }

  TGTer_combinado_horario = class(TGTer)
  public

    pa: TFichaGTer_combinado_horario;
    EstadoTimer: NReal;
    NDisc: integer;
    resCode: TDAofNInt;
    xrpos: TDAofNReal;
    dCFdX, dCFdX_inc, dCFdx_dec: TDAofNReal;
    // afectación del costo futuro por movimiento del estado.

    (******** Estas son las variables de estado ******)
    X_timer_: TDAofNReal;
    // timer que empieza a correr cuando se cierra el ciclo al inicio del paso
    Xs_timer_: TDAofNReal;
    // timer que empieza a correr cuando se cierra el ciclo al final del paso

    horasPurga: NReal; //horas que demora en purgar antes de poder prender la TV
    horasTVPleno: NReal; //horas que demora en llegar a pleno la TV

    costoComb: NReal; //costo por combinar el ciclo
   // disp_cald: NReal; //factor de disponibilidad de las calderas del ciclo
   // tRep

    umbralDespachoTV: NReal;
    factorTV: NReal;   //factor TV/TG si se puede despachar TV o 0 sino

    //variable para saber en que parte del ciclo estoy
    //0 cuando se esta purgando, si purga_TV es 0 y X_timer es 0, el ciclo esta abierto
    //1 cuando se esta subiendo las TG con ciclo cerrado
    //2 cuando puedo estar a pleno
    purga_TV_: TDAofNInt;

    //variable para saber si arranca frio, tibio o caliente
    //0 frio
    //1 tibio
    //2 caliente
    tipo_arranque: TDAofNInt;

    //_SC sin calentar (sin calentar calderas) _CC calentando caldera
    //divido la variable de la potencia de las TG en 2, para diferenciar
    //si se calienta o no la caldera
    BMaxTG_SC, BMaxTG_CC, BMaxTV: NReal; // MW, cota sup de P-Pmin
    PMaxTG: NReal;
    PMinTG: NReal; //MW, mínimo
    PMinTV: NReal; // MW, minimo de las TV

    NmaxTG: integer;
    NMaquinasDespachadasTG, NMaquinasDespachadasTV: integer;

    PdespachTG, PdespachTV: NReal;

    //pendiente con la que sube y baja la temperatura de la caldera
    //    pendTempSubida, pendTempBajada: NReal;
    pendTV_frio, pendTV_tibio, pendTV_caliente: NReal;

    c0TG, c0TV: NReal;
    //Costo en USD/h por máquina por estar operando en el mínimo técnico
    cvTG, cvTV: NReal; //USD/MWh usado para la optimización del paso.
   // cAcople: NReal; //Costo por acoplar

    // resultado de los sorteos de disponibilidad y del mantenimiento programado
    NMaquinasDisponiblesTG, NMaquinasDisponiblesTV: integer;
    NCalderasDisponibles: integer;
    PMaxDisponible_TG, PMaxDisponible_TV: NReal;
    PMaxDisponible_Central: NReal;

    // indices a los tramos de variables
    ivar_ATG_SC, ivar_BTG_SC, ivar_ATG_CC, ivar_BTG_CC,
    ivar_ATV, ivar_BTV, ivar_AAcople, ivar_Xs_timer: integer;

    // indices a las restricciones
    jres_dinTimer, jres_Acople_BTG_SC, jres_Acople_BTG_CC,
    jres_Acople_BTV, jres_Acople_TVxTG, jres_Acople_TimerxBTG_SC,
    jres_Acople_TimerxBTG_CC, jres_extTimer, jres_EMaxPasoDeTiempo,
    jres_Ultima, jaux: integer;

    //Cuanta potencia despacho en cada hora el generador en promedio
    potMedia_despachada: NReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; TonCO2xMWh: NReal; LowCostMustRun,
  CleanDevelopmentMechanism: boolean; xFuenteIdxP: TFuenteAleatoria;
  xBorneIdxP: string; EstadoTimer: NReal);


    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(f: TArchiTexto); override;

    procedure PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos(CatalogoReferencias: TCatalogoReferencias);
      override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    function get_pa_FD(kTipoUnidad: integer): NReal; override;
    function get_pa_TMR(kTipoUnidad: integer): NReal; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;
    procedure Sim_Paso_Fin; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_nvers_TV_indisponible(var ivar, ivae, ires: integer);
    procedure opt_nvers_abierto_purga(var ivar, ivae, ires: integer);
    procedure opt_nvers_combinado(var ivar, ivae, ires: integer);

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_cargue_TV_indisponible(s: TSimplex);
    procedure opt_cargue_abierto_purga(s: TSimplex);
    procedure opt_cargue_combinado(s: TSimplex);

    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja_TV_indisponible(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_abierto_purga(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_transicion(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_combinado(s: TSimplex);

    procedure opt_leerSolucion(s: TSimplex); override;
    procedure opt_leerSolucion_TV_indisponible(s: TSimplex);
    procedure opt_leerSolucion_abierto_purga(s: TSimplex);
    procedure opt_leerSolucion_combinado(s: TSimplex);
    procedure ChequearYActualizarTimers;

    procedure Sim_Cronica_Inicio; override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreVar_TV_indisponible(ivar: integer; var nombre: string): boolean;
    function getNombreVar_abierto_purga(ivar: integer; var nombre: string): boolean;
    function getNombreVar_combinado(ivar: integer; var nombre: string): boolean;

    function getNombreRes(ires: integer; var nombre: string): boolean; override;
    function getNombreRes_TV_indisponible(ires: integer; var nombre: string): boolean;
    function getNombreRes_abierto_purga(ires: integer; var nombre: string): boolean;
    function getNombreRes_combinado(ires: integer; var nombre: string): boolean;

    destructor destroy; override;

    procedure PubliVars; override;
    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    class function TipoFichaLPD: TClaseDeFichaLPD; override;

    procedure EvolucionarEstado; override;

    procedure PosicionarseEnEstrellita; override;
    procedure ActualizarEstadoGlobal(flg_Xs: boolean); override;

    procedure optx_nvxs(var ixr, ixd, iauxr, iauxd: integer); override;
    procedure optx_RegistrarVariablesDeEstado(adminEstados: TAdminEstados); override;

  end;

procedure AlInicio;
procedure AlFinal;

implementation

//------------------------------
// Métodos de TFichaGter_combinado_horario
//==============================
constructor TFichaGTer_combinado_horario.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; PMin_TG, PMax_TG: NReal; cv_min_TG, cv_TG,
  cv_NoCombustible_TG: NReal; disp_TG: NReal; tRepHoras_TG: NReal; PMin_TV,
  PMax_TV: NReal; cv_min_TV, cv_TV, cv_NoCombustible_TV: NReal; disp_TV: NReal;
  tRepHoras_TV: NReal; factorPotenciaTVdivTG: NReal;
  indicePreciosPorCombustible: TFuenteAleatoria;
  bornePreciosPorCombustible: string; HayRestriccionEmaxPasoDeTiempo: boolean;
  EmaxPasoDeTiempo: NReal; PagoPorDisponibilidad_USD_MWh,
  PagoPorEnergia_USD_MWh: NReal; horasPurga: NReal; horasTVPleno: NReal;
  NDisc: integer; costoComb: NReal; disp_cald: NReal; tRepHoras_cald: NReal);

begin
  inherited Create(capa, fecha, periodicidad);

  self.PMin_TG := PMin_TG;
  self.PMax_TG := PMax_TG;
  self.cv_min_TG := cv_min_TG;
  self.cv_TG := cv_TG;
  self.cv_NoCombustible_TG := cv_NoCombustible_TG;
  self.disp_TG := disp_TG;
  self.tRepHoras_TG := tRepHoras_TG;

  self.disp_TV := disp_TV;
  self.tRepHoras_TV := tRepHoras_TV;

  self.PMin_TV := PMin_TV;
  self.PMax_TV := PMax_TV;
  self.cv_min_TV := cv_min_TV;
  self.cv_TV := cv_TV;
  self.cv_NoCombustible_TV := cv_NoCombustible_TV;
  self.flg_TV_OnOffPorPaso := flg_TV_OnOffPorPaso;

  self.factorPotenciaTVdivTG := factorPotenciaTVdivTG;

  self.indicePreciosPorCombustible := indicePreciosPorCombustible;
  self.bornePreciosPorCombustible := bornePreciosPorCombustible;
  self.HayRestriccionEmaxPasoDeTiempo := HayRestriccionEmaxPasoDeTiempo;
  self.EmaxPasoDeTiempo := EmaxPasoDeTiempo;
  self.PagoPorDisponibilidad_USD_MWh := PagoPorDisponibilidad_USD_MWh;
  self.PagoPorEnergia_USD_MWh := PagoPorEnergia_USD_MWh;


  self.horasPurga := horasPurga;
  self.horasTVPleno := horasTVPleno;
  self.NDisc := NDisc;
  self.costoComb := costoComb;
  self.disp_cald := disp_cald;
  self.tRepHoras_cald := tRepHoras_cald;

end;

function TFichaGTer_combinado_horario.Rec: TCosa_RecLnk;
begin
  Result := inherited Rec;
  Result.addCampoDef('PMin_TG', PMin_TG);
  Result.addCampoDef('PMax_TG', PMax_TG);
  Result.addCampoDef('cv_min_TG', cv_min_TG);
  Result.addCampoDef('cv_TG', cv_TG);
  Result.addCampoDef('cv_NoCombustible_TG', cv_NoCombustible_TG, 111);
  Result.addCampoDef('disp_TG', disp_TG);
  Result.addCampoDef('tRepHoras_TG', tRepHoras_TG);
  Result.addCampoDef('disp_TV', disp_TV, 0, 98);
  Result.addCampoDef('tRepHoras_TV', tRepHoras_TV, 0, 98);
  Result.addCampoDef('PMin_CC', PMin_TV, 0, 98);
  Result.addCampoDef('PMax_CC', PMax_TV, 0, 98);
  Result.addCampoDef('cv_min_CC', cv_min_TV, 0, 98);
  Result.addCampoDef('cv_CC', cv_TV, 0, 98);
  Result.addCampoDef('PMin_TV', PMin_TV, 98);
  Result.addCampoDef('PMax_TV', PMax_TV, 98);
  Result.addCampoDef('cv_min_TV', cv_min_TV, 98);
  Result.addCampoDef('cv_TV', cv_TV, 98);
  Result.addCampoDef('cv_NoCombustible_TV', cv_NoCombustible_TV, 111);
  Result.addCampoDef('disp_TV', disp_TV, 98);
  Result.addCampoDef('tRepHoras_TV', tRepHoras_TV, 98);
  Result.addCampoDef('flg_TV_OnOffPorPaso', flg_TV_OnOffPorPaso, 113);
  Result.addCampoDef('factorPotenciaTVdivTG', factorPotenciaTVdivTG);
  Result.addCampoDef_ref('indicePreciosPorCombustible',
    TCosa(indicePreciosPorCombustible), self);
  Result.addCampoDef('bornePreciosPorCombustible', bornePreciosPorCombustible);
  Result.addCampoDef('HayRestriccionEmaxPasoDeTiempo', HayRestriccionEmaxPasoDeTiempo);
  Result.addCampoDef('EmaxPasoDeTiempo', EmaxPasoDeTiempo);
  Result.addCampoDef('PagoPorDisponibilidad_USD_MWh',
    PagoPorDisponibilidad_USD_MWh, 98);
  Result.addCampoDef('PagoPorEnergia_USD_MWh', PagoPorEnergia_USD_MWh, 98);
  Result.addCampoDef('horasPurga', horasPurga, 189);
  Result.addCampoDef('horasTVPleno', horasTVPleno, 189);
  Result.addCampoDef('NDisc', NDisc, 189);
  Result.addCampoDef('costoComb', costoComb, 193);
  Result.addCampoDef('disp_cald', disp_cald, 193);
  Result.addCampoDef('tRepHoras_cald', tRepHoras_cald, 193);
end;

procedure TFichaGTer_combinado_horario.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
  flg_TV_OnOffPorPaso := False;
  cv_NoCombustible_TG := 0.0;
  cv_NoCombustible_TV := 0.0;
end;

procedure TFichaGTer_combinado_horario.AfterRead(f: TArchiTexto);
begin
  inherited AfterRead(f);
  if f.version < 113 then
    flg_TV_OnOffPorPaso := False;

  if f.Version < 98 then
  begin
    PagoPorDisponibilidad_USD_MWh := 0;
    PagoPorEnergia_USD_MWh := 0;
  end;

  if f.Version < 98 then
  begin
    PagoPorDisponibilidad_USD_MWh := 0;
    PagoPorEnergia_USD_MWh := 0;
  end;

  if f.Version < 189 then
  begin
    horasPurga := 0;
    horasTVPleno := 0;
    NDisc := 0;
  end;

  if f.Version < 193 then
  begin
    costoComb := 0;
    disp_cald := 0;
    tRepHoras_cald := 0;
  end;

end;



procedure TFichaGTer_combinado_horario.generarLineaResumen(var archi: TextFile);
begin
  Write(archi, FloatToStrF(PMin_TG, formatoReales, 8, 1), #9,
    //PMín_TG
    FloatToStrF(PMax_TG, formatoReales, 8, 1), #9,
    //PMáx_TG
    FloatToStrF(cv_min_TG, formatoReales, 8, 2), #9,
    //CV_Mín_TG
    FloatToStrF(((cv_min_TG * PMin_TG + cv_TG * (PMax_TG - PMin_TG)) / PMax_TG),
    formatoReales, 8, 2),
    #9,    //CV_Medio_TG
    FloatToStrF(cv_TG, formatoReales, 8, 2), #9,
    //CV_Incremental_TG
    FloatToStrF(disp_TG, formatoReales, 8, 2), #9,
    //FDisp_TG
    '-', #9,
    //Costo Arranque
    '-', #9,
    //Costo Parada
    '-', #9,
    //mínNPasosOn
    '-', #9,
    //mínNPasosOff
    '-', #9,
    //desiciónPasosOnPorCiclo
    '-', #9,
    //desiciónPasosOffPorCiclo
    '-', #9,
    //costoPorCicloOn
    '-', #9);
  //costoPorCicloOff
end;

function TFichaGTer_combinado_horario.infoAd_: string;
begin
  Result := 'PMín_TG= ' + FloatToStrF(PMin_TG, ffGeneral, 10, 1) +
    ' MW, ' + 'PMáx_TG= ' + FloatToStrF(PMax_TG, ffGeneral, 10, 1) +
    ' MW, ' + 'cv_min_TG= ' + FloatToStrF(cv_min_TG, ffGeneral, 10, 1) +
    ' USD/MWh, ' + 'cv_TG= ' + FloatToStrF(cv_TG, ffGeneral, 10, 1) +
    ' USD/MWh, ' + 'fDisp_TG= ' + FloatToStrF(disp_TG, ffGeneral, 10, 2) +
    ' p.u., ' + 'tRep_TG= ' + FloatToStrF(tRepHoras_TG, ffGeneral, 10, 1) + 'h';
end;

destructor TFichaGTer_combinado_horario.destroy;
begin
  inherited destroy;
end;

//------------------------
// Métodos de TGTer_Combinado_horario
//========================
procedure TGTer_combinado_horario.dump_Variables(var f: TextFile; charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'BMaxTG_SC[MW]= ', FloatToStrF(BMaxTG_SC, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'BMaxTG_CC[MW]= ', FloatToStrF(BMaxTG_CC, ffFixed, 10, 3));
  writeln(f);
end;

class function TGTer_combinado_horario.TipoFichaLPD: TClaseDeFichaLPD;
begin
  Result := TFichaGTer_combinado_horario;
end;


procedure TGTer_combinado_horario.EvolucionarEstado;
var
  i: integer;
begin
  inherited EvolucionarEstado;
  for i := 0 to NmaxTG - 1 do
  begin
    X_timer_[i] := Xs_timer_[i];
  end;
end;

procedure TGTer_combinado_horario.PosicionarseEnEstrellita;
var
  i: integer;
begin
  inherited PosicionarseEnEstrellita;
  for i := 0 to NmaxTG - 1 do
  begin
    X_timer_[i] := globs.CF.xr[ixr + i];
  end;

end;

procedure TGTer_combinado_horario.ActualizarEstadoGlobal(flg_Xs: boolean);
var
  i: integer;
begin
  inherited ActualizarEstadoGlobal(flg_Xs);
  for i := 0 to NmaxTG - 1 do
  begin
  globs.CF.xr[ixr + i] := X_timer_[i];
  end;
end;

procedure TGTer_combinado_horario.optx_nvxs(var ixr, ixd, iauxr, iauxd: integer);
begin
  inherited optx_nvxs(ixr, ixd, iauxr, iauxd);
  self.ixr := ixr;
  ixr := ixr + NmaxTG;
end;

procedure TGTer_combinado_horario.optx_RegistrarVariablesDeEstado(
  adminEstados: TAdminEstados);
var
  i: integer;
begin
  inherited optx_RegistrarVariablesDeEstado(adminEstados);

  for i := 0 to NmaxTG - 1 do
  begin
  adminEstados.Registrar_Continua(
    self.ixr + i,
    0,
    5,
    pa.NDisc,
    nombre + '_timer_' + IntToStr(i), // nombre de la variable
    'h' // unidades
    );
  end;


end;

constructor TGTer_combinado_horario.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean; xFuenteIdxP: TFuenteAleatoria;
  xBorneIdxP: string; EstadoTimer: NReal);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, lpd, nodo,
    flg_CalcularGradienteDeInversion, TonCO2xMWh, LowCostMustRun,
    CleanDevelopmentMechanism,
    xFuenteIdxP, xBorneIdxP);
  self.EstadoTimer := EstadoTimer;
end;

function TGTer_combinado_horario.Rec: TCosa_RecLnk;
begin
  Result := inherited Rec;
  Result.addCampoDef('EstadoTimer', EstadoTimer, 190);
end;

procedure TGTer_combinado_horario.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
end;

procedure TGTer_combinado_horario.AfterRead(f: TArchiTexto);
begin
  inherited AfterRead(f);
end;

procedure TGTer_combinado_horario.PrepararMemoria(Catalogo: TCatalogoReferencias;
  globs: TGlobs);
begin
  inherited prepararMemoria(Catalogo, globs);
  NMaquinasDespachadasTG := 0;
  NMaquinasDespachadasTV := 0;
  NmaxTG := 2;//paUnidades.nUnidades_Operativas[0]; //PRUEBA
  SetLength(X_timer_, NmaxTG);
  SetLength(Xs_timer_, NmaxTG);
  SetLength(purga_TV_, NmaxTG);
  SetLength(dCFdX, NmaxTG);
  SetLength(dCFdx_dec, NmaxTG);
  SetLength(dCFdX_inc, NmaxTG);
  SetLength(xrpos, NmaxTG);
  SetLength(resCode, NmaxTG);
end;

procedure TGTer_combinado_horario.RegistrarParametrosDinamicos(
  CatalogoReferencias: TCatalogoReferencias);
var
  i: integer;
  ficha: TFichaGTer_combinado_horario;
begin
  inherited registrarParametrosDinamicos(CatalogoReferencias);
  lpd.expandirFichas(CatalogoReferencias, globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
  for i := 0 to lpd.Count - 1 do
  begin
    ficha := TFichaGTer_combinado_horario(lpd[i]);
    if ficha.indicePreciosPorCombustible <> nil then
      ficha.nroBornePreciosPorCombustible :=
        ficha.indicePreciosPorCombustible.IdBorne(ficha.bornePreciosPorCombustible);
  end;
end;

function TGTer_combinado_horario.PotenciaFirme: NReal;
var
  PTG, PTVa, PTVb: NReal;
begin
  PTG := paUnidades.nUnidades_Operativas[0] * pa.PMax_TG * pa.disp_TG;
  PTVa := paUnidades.nUnidades_Operativas[1] * pa.PMax_TV * pa.disp_TV;
  PTVb := PTG * pa.factorPotenciaTVdivTG;
  Result := PTG + min(PTVb, PTVa);
end;

function TGTer_combinado_horario.InfoAd_: string;
begin
  Result := '';
end;

class function TGTer_combinado_horario.DescClase: string;
begin
  Result := rsGeneradorTermicoCombinadoHorario;
end;

function TGTer_combinado_horario.get_pa_FD(kTipoUnidad: integer): NReal;
begin
  if kTipoUnidad = 0 then
    Result := pa.disp_TG
  else
    Result := pa.disp_TV;
end;

function TGTer_combinado_horario.get_pa_TMR(kTipoUnidad: integer): NReal;
begin
  if kTipoUnidad = 0 then
    Result := pa.tRepHoras_TG
  else
    Result := pa.tRepHoras_TV;
end;


procedure TGTer_combinado_horario.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
    NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
    NCalderasDisponibles := NMaquinasDisponiblesTG;
    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
    PMaxDisponible_TG * pa.factorPotenciaTVdivTG);
  end
  else
  if sortear then
  begin
    if (globs.EstadoDeLaSala = CES_OPTIMIZANDO) then
    begin
      ActualizarProbabilidadesReparacionYRotura_(1, 0, 0);
      NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
      ActualizarProbabilidadesReparacionYRotura_(1, 0, 1);
      NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
      NCalderasDisponibles := NMaquinasDisponiblesTG;
    end
    else
    begin
      ActualizarProbabilidadesReparacionYRotura_(pa.disp_TG, pa.tRepHoras_TG, 0);
      NMaquinasDisponiblesTG := Sorteos_RepRotUnidades_k(0);
      ActualizarProbabilidadesReparacionYRotura_(pa.disp_TV, pa.tRepHoras_TV, 1);
      NMaquinasDisponiblesTV := Sorteos_RepRotUnidades_k(1);
      ActualizarProbabilidadesReparacionYRotura_(pa.disp_cald, pa.tRepHoras_cald, 1);
      NCalderasDisponibles := Sorteos_RepRotUnidades_k(2);
    end;
    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
    PMaxDisponible_TG * pa.factorPotenciaTVdivTG);
  end
  else
  begin
    NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
    NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
    NCalderasDisponibles := paUnidades.nUnidades_Operativas[2];

    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG * pa.disp_TG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
    PMaxDisponible_TG * pa.factorPotenciaTVdivTG) * pa.disp_TV;
  end;

  PMaxDisponible_Central := PMaxDisponible_TG + PMaxDisponible_TV;
end;

procedure TGTer_combinado_horario.PrepararPaso_ps;
var
  indice: NReal;
  i: integer;
begin
  {$IFDEF dbgc}
  if ( globs.EstadoDeLaSala =   CES_SIMULANDO) and (globs.kPaso_Sim = 19 ) then
     writeln('a');
  {$ENDIF}

  PMinTG := pa.PMin_TG;
  BMaxTG_SC := pa.PMax_TG - pa.PMin_TG;
  BMaxTG_CC := BMaxTG_SC;
  PMinTV := pa.PMin_TV;
  BMaxTV := pa.PMax_TV - pa.PMin_TV;
  PMaxTG := pa.PMax_TG;
  horasPurga := pa.horasPurga;
  horasTVPleno := pa.horasTVPleno;
  NDisc := pa.NDisc;
  costoComb := pa.costoComb;

//  SetLength(X_timer_, NMaquinasDisponiblesTG);
//  SetLength(Xs_timer_, NMaquinasDisponiblesTG);
//  SetLength(purga_TV_, NMaquinasDisponiblesTG);

  //pendTV es la pendiente para prender a pleno la TV (que depende de las TG a traves de factorTV)
  pendTV_frio := (PMaxTG - PMinTG) / (horasTVPleno);
  pendTV_tibio := (PMaxTG - PMinTG) / (horasTVPleno);
  pendTV_caliente := 1;

  if pa.indicePreciosPorCombustible <> nil then
  begin
    indice := pa.indicePreciosPorCombustible.bornera[pa.nroBornePreciosPorCombustible];
    c0TG := pa.PMin_TG * (pa.cv_min_TG * indice + pa.cv_NoCombustible_TG);
    //este dsp va multiplicado por algo adimensionado (A)
    cvTG := pa.cv_TG * indice + pa.cv_NoCombustible_TG;
    //este va por algo con dimension MW (coef B)
  end
  else
  begin
    c0TG := pa.PMin_TG * (pa.cv_min_TG + pa.cv_NoCombustible_TG);
    cvTG := pa.cv_TG + pa.cv_NoCombustible_TG;
  end;
  c0TV := pa.PMin_TV * (pa.cv_min_TV + pa.cv_NoCombustible_TV);
  cvTV := pa.cv_TV + pa.cv_NoCombustible_TV;

  for i := 0 to NmaxTG - 1 do
  begin
    if (X_timer_[i]) < (horasPurga) then //tiempo purga
    begin
      factorTV := 0;
      purga_TV_[i] := 0;
    end
    else
    begin
      if (X_timer_[i]) < (horasTVPleno + horasPurga - 1) then //prendiendo TV despacito
      begin
        factorTV := pa.factorPotenciaTVdivTG;
        purga_TV_[i] := 1;
      end
      else
      begin
        factorTV := pa.factorPotenciaTVdivTG;
        purga_TV_[i] := 2;  //TV a pleno
      end;
    end;
  end;

  // Cálculo del al afectación del futuro por movimiento de estado.
  for i := 0 to NmaxTG - 1 do
  begin
    globs.CF.devxr_continuo(ixr, globs.kPaso_Opt + 1, dCFdX_Inc[i],
          dCFdX_Dec[i], resCode[i], xrpos[i] );
    dCFdX[i] := ( dCFdX_Inc[i] + dCFdX_dec[i] ) / 2;

//    if X_timer_[i] <= 5 then
//    begin
//      dCFdX := globs.CF.deltaCosto_vxr_continuo(ixr + i, globs.kPaso_Opt + 1, 1);
//    end
//    else
//    begin
//      dCFdX := - globs.CF.deltaCosto_vxr_continuo(ixr + i, globs.kPaso_Opt + 1, -5);
//    end;

  end;
end;

procedure TGTer_combinado_horario.Sim_Paso_Fin;
var
  iposte: integer;
begin
  if NMaquinasDisponiblesTG > 0 then
  begin
    for iposte := 0 to high(P) do
      potMedia_despachada := potMedia_despachada + P[iposte] * globs.durpos[iposte];
    potMedia_despachada := potMedia_despachada * globs.invHorasDelPaso;
  end
  else
  begin
    potMedia_despachada := 0;
  end;
end;

procedure TGTer_combinado_horario.opt_nvers(var ivar, ivae, ires: integer);
begin
  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;
  if ((NMaquinasDisponiblesTV = 0)  or (NCalderasDisponibles = 0)) then
  begin
    opt_nvers_TV_indisponible(ivar, ivae, ires);
  end
  else
  begin
    opt_nvers_combinado(ivar, ivae, ires);
  end;

  {
  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_nvers_TV_indisponible(ivar, ivae, ires);
  end
  else
  begin
    case purga_TV_[0] of
      0:
      begin
        opt_nvers_abierto_purga(ivar, ivae, ires);
      end;
      1, 2:
      begin
        opt_nvers_combinado(ivar, ivae, ires);
      end;
    end;
  end;
  }

end;

procedure TGTer_combinado_horario.opt_nvers_TV_indisponible(var ivar, ivae,
  ires: integer);
begin

  //variables ATG_SC_i, BTG_SC_i, Xs_Timer_i
  //Por cada TG hay ATG_SC y Xs_timer
  Self.ivar := ivar;
  ivar := ivar + 2*NmaxTG + NmaxTG;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_Xs_timer := ivar_BTG_SC + NmaxTG + 1;

  // Las variables ATG_SC son enteras
  Self.ivae := ivae;
  ivae := ivae + NmaxTG;

  Self.ires := ires;
  ires := ires + NmaxTG + NmaxTG + NmaxTG;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_SC, BTV < (PMax-PMin)*ATV, XTemp < Atemp*temp_max
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + NmaxTG;
  jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;
  jaux := jres_Acople_TimerxBTG_SC + NmaxTG;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

   {
    //variables ATG_SC, BTG_SC, Xs_Timer
  Self.ivar := ivar;
  ivar := ivar + 3;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_Xs_timer := ivar_BTG_SC + 1;

  // Las variables ATG_SC, ATG_CC y Aacople son enteras
  Self.ivae := ivae;
  ivae := ivae + 1;

  Self.ires := ires;
  ires := ires + 3;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_CC, BTV < (PMax-PMin)*ATV, Xs_Timer = AAcople*(X_timer + durPaso)
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;

  jaux := jres_Acople_TimerxBTG_SC;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;
    }
end;

procedure TGTer_combinado_horario.opt_nvers_abierto_purga(var ivar, ivae,
  ires: integer);
begin

  //variables ATG_SC, BTG_SC, ATG_CC, BTG_CC, AAcople, Xs_Timer
  Self.ivar := ivar;
  ivar := ivar + 5;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_ATG_CC := ivar_BTG_SC + 1;
 // ivar_BTG_CC := ivar_ATG_CC + 1;

  ivar_AAcople := ivar_ATG_CC + 1;
  ivar_Xs_timer := ivar_AAcople + 1;

  // Las variables ATG_SC, ATG_CC y Aacople son enteras
  Self.ivae := ivae;
  ivae := ivae + 3;

  Self.ires := ires;
  ires := ires + 4;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_CC, BTV < (PMax-PMin)*ATV, Xs_Timer = AAcople*(X_timer + durPaso)
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;
  jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + 1;

  jaux := jres_Acople_TimerxBTG_CC;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

end;

procedure TGTer_combinado_horario.opt_nvers_combinado(var ivar, ivae,
  ires: integer);
begin

  //variables ATG_SC_i, BTG_SC_i, ATG_CC_i, BTG_CC_i, ATV, BTV, AAcople_i, Xs_Timer_i
  //Por cada TG hay ATG_SC ATG_CC AAcople y Xs_timer
  Self.ivar := ivar;
  ivar := ivar + 2*NmaxTG + 2*NmaxTG + 2 + 2*NmaxTG;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_ATG_CC := ivar_BTG_SC + NmaxTG + 1;
  ivar_BTG_CC := ivar_ATG_CC + 1;

  ivar_ATV := ivar_BTG_CC + NmaxTG + 1;
  ivar_BTV := ivar_ATV + 1;

  ivar_AAcople := ivar_BTV + 1;
  ivar_Xs_timer := ivar_AAcople + NmaxTG;

  // Las variables ATG_SC, ATG_CC, ATV y ATemp son enteras
  Self.ivae := ivae;
  ivae := ivae + 2*NmaxTG + 2*NmaxTG + 2*NmaxTG;

  Self.ires := ires;
  ires := ires + NmaxTG + NmaxTG + NmaxTG + 2 + NmaxTG + NmaxTG;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_SC, BTV < (PMax-PMin)*ATV, XTemp < Atemp*temp_max
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + NmaxTG;
  jres_Acople_BTG_CC := jres_Acople_BTG_SC + NmaxTG;
  jres_Acople_BTV := jres_Acople_BTG_CC + NmaxTG;
  jres_Acople_TVxTG := jres_Acople_BTV + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_TVxTG + 1;
  jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + NmaxTG;
  jaux := jres_Acople_TimerxBTG_CC + NmaxTG;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

end;

procedure TGTer_combinado_horario.opt_cargue(s: TSimplex);
begin

  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;
  if ((NMaquinasDisponiblesTV = 0) or (NCalderasDisponibles = 0)) then
  begin
    opt_cargue_TV_indisponible(s);
  end
  else
  begin
    opt_cargue_combinado(s);
  end;

  {
  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_cargue_TV_indisponible(s);
  end
  else
  begin
    case purga_TV_[0] of
      0:
      begin
        opt_cargue_abierto_purga(s);
      end;
      1, 2:
      begin
        opt_cargue_combinado(s);
      end;
    end;
  end;
  }

end;

procedure TGTer_combinado_horario.opt_cargue_TV_indisponible(s: TSimplex);
var
   inodores: integer;
   i: integer;
begin
  inodores := nodo.ires;
  // aporte a las restricciones del nodo
  for i := 0 to NmaxTG - 1 do
  begin
  s.pon_e(inodores, ivar_ATG_SC + 2*i, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC + 2*i, 1); // coef de la B TG sin calentar caldera
  end;

  //restriccion de timer
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_dinTimer + i, ivar_Xs_timer + i, -1);
    s.pon_e(jres_dinTimer + i, s.nc + i, X_timer_[i] - globs.HorasDelPaso);
  end;

  // coef B de las TG
  for i := 0 to NmaxTG - 1 do
  begin
  s.pon_e(jres_Acople_BTG_SC + i, ivar_ATG_SC + 2*i, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC + i, ivar_BTG_SC + 2*i, -1);
  end;

  //restriccion PTGSC menor que el max (cuando esta Aacople apagado)
  for i := 0 to NmaxTG - 1 do
  begin
  s.pon_e(jres_Acople_TimerxBTG_SC + i, ivar_ATG_SC + 2*i, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC + i, s.nc, 1 + 0.1);
  end;


  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(s.nf, ivar_ATG_SC + 2*i, -c0TG * globs.HorasDelPaso);
    s.pon_e(s.nf, ivar_BTG_SC + 2*i, -cvTG * globs.HorasDelPaso);
  end;

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  for i := 0 to NmaxTG - 1 do
  begin
  s.pon_e(s.nf, ivar_Xs_timer + i, -dCFdX[i]);
  s.acum_e(s.nf, s.nc, dCFdX[i] * X_timer_[i]);
  end;
  {$ENDIF}
  {
  inodores := nodo.ires;
  // aporte a las restricciones del nodo
  s.pon_e(inodores, ivar_ATG_SC, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC, 1); // coef de la B TG sin calentar caldera


  //restriccion de timer
  s.pon_e(jres_dinTimer, ivar_Xs_timer, -1);


  // coef B de las TG
  s.pon_e(jres_Acople_BTG_SC, ivar_ATG_SC, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC, ivar_BTG_SC, -1);

  //restriccion PTGSC menor que el max (cuando esta AAcople apagado)
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_ATG_SC, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC, s.nc, NMaquinasDisponiblesTG + 0.1);

  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  s.pon_e(s.nf, ivar_BTG_SC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_SC, -c0TG * globs.HorasDelPaso);
  //  s.pon_e(s.nf, ivar_AAcople, -100 * globs.HorasDelPaso);

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  {s.pon_e(s.nf, ivar_Xs_timer, -dCFdX);
  s.acum_e(s.nf, s.nc, dCFdX * X_timer_[0]);
  {$ENDIF}

end;

procedure TGTer_combinado_horario.opt_cargue_abierto_purga(s: TSimplex);
var
   inodores: integer;
begin
    inodores := nodo.ires;
  // aporte a las restricciones del nodo
  s.pon_e(inodores, ivar_ATG_SC, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC, 1); // coef de la B TG sin calentar caldera
  s.pon_e(inodores, ivar_ATG_CC, PMinTG);   // coef A TG calentando caldera
//  s.pon_e(inodores, ivar_BTG_CC, 1); // coef de la B TG calentando caldera

 // jres_tempXBTG := self.ires;
 // jres_Acople_BTG_SC := jres_tempXBTG + 1;
 // jres_Acople_BTG_CC := jres_Acople_BTG_SC + 1;
 // jres_Acople_BTV := jres_Acople_BTG_CC + 1;
 // jres_Acople_TVxTG := jres_Acople_BTV + 1;
 // jres_Acople_tempxBTG_SC := jres_Acople_TVxTG + 1;
 // jres_Acople_tempxBTG_CC := jres_Acople_tempxBTG_SC +1;

  //restriccion de timer
  s.pon_e(jres_dinTimer, ivar_AAcople, X_timer_[0] + globs.HorasDelPaso);
  s.pon_e(jres_dinTimer, ivar_Xs_timer, -1);
//  s.pon_e(jres_dinTimer, s.nc, 0.1);

  // coef B de las TG
  s.pon_e(jres_Acople_BTG_SC, ivar_ATG_SC, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC, ivar_BTG_SC, -1);

  //restriccion PTGSC menor que el max (cuando esta AAcople apagado)
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_ATG_SC, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_AAcople, -(NMaquinasDisponiblesTG));
  s.pon_e(jres_Acople_TimerxBTG_SC, s.nc, NMaquinasDisponiblesTG + 0.1);

  //restriccion PTGCC menor que el max (cuando esta AAcople prendido)
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_ATG_CC, -1);
  //  s.pon_e(jres_Acople_tempxBTG_CC, ivar_BTG_CC, -1);
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_AAcople, NMaquinasDisponiblesTG);

  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  s.pon_e(s.nf, ivar_BTG_SC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_SC, -c0TG * globs.HorasDelPaso);
 // s.pon_e(s.nf, ivar_BTG_CC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_CC, -c0TG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_AAcople, -costoComb * globs.HorasDelPaso);

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  s.pon_e(s.nf, ivar_Xs_timer, -dCFdX[0]);
  s.acum_e(s.nf, s.nc, dCFdX[0] * X_timer_[0]);
  {$ENDIF}

end;

procedure TGTer_combinado_horario.opt_cargue_combinado(s: TSimplex);
var
   inodores: integer;
   i: integer;
begin

  inodores := nodo.ires;

  // aporte a las restricciones del nodo
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(inodores, ivar_ATG_SC + 2*i, PMinTG); // coef A TG sin calentar caldera
    s.pon_e(inodores, ivar_BTG_SC + 2*i, 1); // coef de la B TG sin calentar caldera
    s.pon_e(inodores, ivar_ATG_CC + 2*i, PMinTG);   // coef A TG calentando caldera
    s.pon_e(inodores, ivar_BTG_CC + 2*i, 1); // coef de la B TG calentando caldera
  end;

  s.pon_e(inodores, ivar_ATV, PMinTV);   // coef A TG calentando caldera
  s.pon_e(inodores, ivar_BTV, 1); // coef de la B TG calentando caldera

 // jres_tempXBTG := self.ires;
 // jres_Acople_BTG_SC := jres_tempXBTG + 1;
 // jres_Acople_BTG_CC := jres_Acople_BTG_SC + 1;
 // jres_Acople_BTV := jres_Acople_BTG_CC + 1;
 // jres_Acople_TVxTG := jres_Acople_BTV + 1;
 // jres_Acople_tempxBTG_SC := jres_Acople_TVxTG + 1;
 // jres_Acople_tempxBTG_CC := jres_Acople_tempxBTG_SC +1;

  //restriccion de timer
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_dinTimer + i, ivar_AAcople + i, 2*globs.HorasDelPaso);
    s.pon_e(jres_dinTimer + i, ivar_Xs_timer + i, -1);
    s.pon_e(jres_dinTimer + i, s.nc, X_timer_[i] - globs.HorasDelPaso);
  end;

  // coef B de las TG
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_Acople_BTG_SC + i, ivar_ATG_SC + 2*i, BMaxTG_SC);
    s.pon_e(jres_Acople_BTG_SC + i, ivar_BTG_SC + 2*i, -1);
  end;

  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_Acople_BTG_CC + i, ivar_ATG_CC + 2*i, BMaxTG_CC);
    s.pon_e(jres_Acople_BTG_CC + i, ivar_BTG_CC + 2*i, -1);
  end;

  // coef B[iposte] de las TV
  s.pon_e(jres_Acople_BTV, ivar_ATV, BMaxTV);
  s.pon_e(jres_Acople_BTV, ivar_BTV, -1);


  // restricción de acople de las centrales TG y TV
  if not hayForzamientos then
  begin
    for i := 0 to NmaxTG - 1 do
    begin
    s.pon_e(jres_Acople_TVxTG, ivar_ATG_CC + 2*i, factorTV * PMinTG);
    s.pon_e(jres_Acople_TVxTG, ivar_BTG_CC + 2*i, factorTV);
    end;
  end
  else
  begin
    for i := 0 to NmaxTG - 1 do
    begin
    s.pon_e(jres_Acople_TVxTG, ivar_ATG_CC + 2*i, factorTV * PMinTG * 1.3);
    s.pon_e(jres_Acople_TVxTG, ivar_BTG_CC + 2*i, factorTV * 1.3);
    end;
  end;
  s.pon_e(jres_Acople_TVxTG, ivar_ATV, -PMinTV);
  s.pon_e(jres_Acople_TVxTG, ivar_BTV, -1);

  //restriccion PTGSC menor que el max (cuando esta Aacople apagado)
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_Acople_TimerxBTG_SC + i, ivar_ATG_SC + 2*i, -1);
    s.pon_e(jres_Acople_TimerxBTG_SC + i, ivar_AAcople + i, -1);
    s.pon_e(jres_Acople_TimerxBTG_SC + i, s.nc, 1 + 0.1);
  end;

  //restriccion PTGCC menor que el max (cuando esta Atemp prendido)
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(jres_Acople_TimerxBTG_CC + i, ivar_ATG_CC + 2*i, -1);
    //  s.pon_e(jres_Acople_tempxBTG_CC, ivar_BTG_CC, -1);
    s.pon_e(jres_Acople_TimerxBTG_CC + i, ivar_AAcople + i, 1);
  end;



  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(s.nf, ivar_ATG_SC + 2*i, -c0TG * globs.HorasDelPaso);
    s.pon_e(s.nf, ivar_BTG_SC + 2*i, -cvTG * globs.HorasDelPaso);
    s.pon_e(s.nf, ivar_BTG_CC + 2*i, -cvTG * globs.HorasDelPaso);
    s.pon_e(s.nf, ivar_ATG_CC + 2*i, -c0TG * globs.HorasDelPaso);
    s.pon_e(s.nf, ivar_AAcople + i, -costoComb * globs.HorasDelPaso);
  end;
  s.pon_e(s.nf, ivar_BTV, -cvTV * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATV, -c0TV * globs.HorasDelPaso);


  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  for i := 0 to NmaxTG - 1 do
  begin
    s.pon_e(s.nf, ivar_Xs_timer + i, -dCFdX[i]);
    s.acum_e(s.nf, s.nc, dCFdX[i] * X_timer_[i]);
  end;
  {$ENDIF}
end;


procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  cotaBTG_CC: NReal;
begin

  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  if ((NMaquinasDisponiblesTV = 0) or (NCalderasDisponibles = 0)) then
  begin
    opt_fijarRestriccionesDeCaja_TV_indisponible(s);
  end
  else
  begin
    opt_fijarRestriccionesDeCaja_combinado(s);
  end;
  {
  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_fijarRestriccionesDeCaja_TV_indisponible(s);
  end
  else
  begin
    if (not hayForzamientos) then
    begin
      // Restricciones de caja de las B (Esto no es necesario pues las restricciones adicionales
      // obligan que estas se cumplan. Por eso le pongo el 1.1 para dejarle la caja un poco floja
      case purga_TV_[0] of
        0:
        begin
           opt_fijarRestriccionesDeCaja_abierto_purga(s);
        end;
        1:
        begin
           opt_fijarRestriccionesDeCaja_transicion(s);
        end;
        2:
        begin
           opt_fijarRestriccionesDeCaja_combinado(s);
        end;
      end;
    end;
  end;

  }

end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_TV_indisponible(
  s: TSimplex);
var
   i: integer;
begin
   for i := 0 to NmaxTG - 1 do
   begin
     s.cota_sup_set(ivar_Xs_timer + i, 5 * 3);
     s.FijarRestriccionIgualdad(jres_dinTimer + i);

     s.set_entera(ivae + i, ivar_ATG_SC + NmaxTG * i, 1);

    if (not hayForzamientos) then
    begin
     s.cota_sup_set(ivar_BTG_SC + NmaxTG * i, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
    end;
  end;

  {
  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  s.cota_sup_set(ivar_Xs_timer, 5 * 2);
  s.FijarRestriccionIgualdad(jres_dinTimer);

  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);

  if (not hayForzamientos) then
  begin
    s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  end;
   }

end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_abierto_purga(
  s: TSimplex);
begin

  s.cota_sup_set(ivar_Xs_timer, 5 * 2);
  s.FijarRestriccionIgualdad(jres_dinTimer);
  s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC);

  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);
  s.set_entera(ivae + 1, ivar_ATG_CC, NMaquinasDisponiblesTG + 1);
  s.set_entera(ivae + 2, ivar_AAcople, 1);

//  if X_timer > AsumaCero then
//     s.FijarVariable(ivar_ATG_CC, NMaquinasDisponiblesTG);

  if (not hayForzamientos) then
  begin
    s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  end;


end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_transicion(
  s: TSimplex);
var
   cotaBTG_CC: NReal;
begin
  s.cota_sup_set(ivar_Xs_timer, 5 * 3);
  s.FijarRestriccionIgualdad(jres_dinTimer);
  s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC);


  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);
  s.set_entera(ivae + 1, ivar_ATG_CC, NMaquinasDisponiblesTG + 1);
  s.set_entera(ivae + 2, ivar_ATV, NMaquinasDisponiblesTV);
  s.set_entera(ivae + 3, ivar_AAcople, 1);

  if (not hayForzamientos) then
  begin
  s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  cotaBTG_CC := Min((pendTV_frio * (X_timer_[0] - horasPurga) + PMinTG) *
    NMaquinasDisponiblesTG, BMaxTG_CC);
  s.cota_sup_set(ivar_BTG_CC, cotaBTG_CC);
  s.cota_sup_set(ivar_BTV, BMaxTV * NMaquinasDisponiblesTV * 1.1);
  end;


end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_combinado(
  s: TSimplex);
var
   i: integer;
   cotaBTG_CC, pend_TV: NReal;
begin
   for i := 0 to NmaxTG - 1 do
   begin
     s.cota_sup_set(ivar_Xs_timer + i, 5 * 3);
     s.FijarRestriccionIgualdad(jres_dinTimer + i);
     s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC + i);

     s.set_entera(ivae + i, ivar_ATG_SC + NmaxTG * i, 1);
     s.set_entera(ivae + i + NmaxTG, ivar_ATG_CC + NmaxTG * i, 1);
     s.set_entera(ivae + 3 + NmaxTG, ivar_AAcople +  i, 1);

    if (not hayForzamientos) then
    begin
     s.cota_sup_set(ivar_BTG_SC + NmaxTG * i, BMaxTG_SC * 1.1);
     if X_timer_[i] >= (horasPurga) then
        cotaBTG_CC := Min((pendTV_frio * (X_timer_[i] - horasPurga) + PMinTG), BMaxTG_CC)
     else
        cotaBTG_CC := 0;

     s.cota_sup_set(ivar_BTG_CC + NmaxTG * i, cotaBTG_CC);
    end;
  end;

  s.FijarRestriccionIgualdad(jres_Acople_TVxTG);
  s.set_entera(ivae + 2 + NmaxTG, ivar_ATV, NMaquinasDisponiblesTV);
  s.cota_sup_set(ivar_BTV, BMaxTV * NMaquinasDisponiblesTV * 1.1);
end;


procedure TGTer_combinado_horario.opt_leerSolucion(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;
  i: integer;
begin

  Ingreso_PorDisponibilidad_ :=
    PMaxDisponible_Central * pa.PagoPorDisponibilidad_USD_MWh * globs.HorasDelPaso;

  CostoDirectoDelPaso :=0;
  Ingreso_PorEnergia_:= 0;

  if NMaquinasDisponiblesTG = 0 then
  begin
    vclear(P);
    NMaquinasDespachadasTG := 0;
    NMaquinasDespachadasTV := 0;
    for i := 0 to NmaxTG - 1 do
      Xs_timer_[i] := 0;
    exit;
  end
  else if ((NMaquinasDisponiblesTV = 0) or (NCalderasDisponibles = 0)) then
  begin
     opt_leerSolucion_TV_indisponible(s)
  end
  else
  begin
    opt_leerSolucion_combinado(s);
  end;

  if globs.EstadoDeLaSala = CES_SIMULANDO then
     ChequearYActualizarTimers;

  {
  if NMaquinasDisponiblesTG = 0 then
  begin
    vclear(P);
    NMaquinasDespachadasTG := 0;
    NMaquinasDespachadasTV := 0;
    Xs_timer_[0] := 0;
    exit;
  end
  else if NMaquinasDisponiblesTV = 0 then
  begin
     opt_leerSolucion_TV_indisponible(s);
  end
  else
  begin
    case purga_TV_[0] of
      0:
      begin
         opt_leerSolucion_abierto_purga(s);
      end;
      1,2:
      begin
         opt_leerSolucion_combinado(s);
      end;
    end;
  end;
   }

end;

procedure TGTer_combinado_horario.opt_leerSolucion_TV_indisponible(s: TSimplex);
var
  ATG_SC: integer;
  BTG_SC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;
  i: integer;
begin

 PDelPaso := 0;
 EnergiaDespachada := 0;
 NMaquinasDespachadasTG := 0;
 for i := 0 to NmaxTG - 1 do
 begin
  Xs_timer_[i] :=  s.xval( ivar_Xs_timer + i);

  if Xs_timer_[i] >= (5*0.9) then
     Xs_timer_[i] := 5
  else
  if Xs_timer_[i] <= AsumaCero then
     Xs_timer_[i] := 0;

  ATG_SC := trunc(s.xval(ivar_ATG_SC + 2*i) + 0.2);

  NMaquinasDespachadasTG := NMaquinasDespachadasTG + ATG_SC;

  if ATG_SC > 0 then
  begin
    BTG_SC := s.xval(ivar_BTG_SC + 2*i);
    PDelPaso := PDelPaso + BTG_SC + ATG_SC * PMinTG;
  end
  else
    BTG_SC := 0;

  if NMaquinasDespachadasTG > 0 then
  begin
    CostoDirectoDelPaso := CostoDirectoDelPaso + ( BTG_SC * cvTG + ATG_SC * c0TG) * globs.HorasDelPaso;  //100 * AAcople +
  end
  else
    CostoDirectoDelPaso := CostoDirectoDelPaso;

  end;

  PdespachTG := PDelPaso;

  PdespachTV := PDelPaso - PdespachTG;


  EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
  P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

  Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;
  {
 Xs_timer_[0] :=  s.xval( ivar_Xs_timer );
 if Xs_timer_[0] >= (5*0.9) then
      Xs_timer_[0] := 5
 else
 if Xs_timer_[0] <= AsumaCero then
      Xs_timer_[0] := 0;

 EnergiaDespachada := 0;

   ATG_SC := trunc(s.xval(ivar_ATG_SC) + 0.2);

   NMaquinasDespachadasTG := ATG_SC;
   NMaquinasDespachadasTV := 0;

   PDelPaso := 0;
   if ATG_SC > 0 then
   begin
     BTG_SC := s.xval(ivar_BTG_SC);
     PDelPaso := BTG_SC + ATG_SC * PMinTG;
   end
   else
     BTG_SC := 0;

   EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
   P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

   PdespachTG := PDelPaso;

   if NMaquinasDespachadasTG > 0 then
   begin
     CostoDirectoDelPaso := ( BTG_SC * cvTG + ATG_SC * c0TG) *
       globs.HorasDelPaso;  //100 * AAcople +
   end
   else
     CostoDirectoDelPaso := 0;

 Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;
  }
end;

procedure TGTer_combinado_horario.opt_leerSolucion_abierto_purga(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;

begin
 Xs_timer_[0] :=  s.xval( ivar_Xs_timer );
 if Xs_timer_[0] >= (5*0.9) then
      Xs_timer_[0] := 5
 else
 if Xs_timer_[0] <= AsumaCero then
      Xs_timer_[0] := 0;

 EnergiaDespachada := 0;

   ATG_SC := trunc(s.xval(ivar_ATG_SC) + 0.2);
   ATG_CC := trunc(s.xval(ivar_ATG_CC) + 0.2);
   AAcople := trunc(s.xval(ivar_AAcople) + 0.2);

   NMaquinasDespachadasTG := ATG_SC + ATG_CC;
   NMaquinasDespachadasTV := 0;

   PDelPaso := 0;
   if ATG_SC > 0 then
   begin
     BTG_SC := s.xval(ivar_BTG_SC);
     PDelPaso := BTG_SC + ATG_SC * PMinTG;
   end
   else
     BTG_SC := 0;

   if ATG_CC > 0 then
   begin
     PDelPaso := ATG_CC * PMinTG;
   end;

   EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
   P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

   PdespachTG := PDelPaso;

   if NMaquinasDespachadasTG > 0 then
   begin
     CostoDirectoDelPaso := ( BTG_SC * cvTG + ATG_SC * c0TG + ATG_CC * c0TG + AAcople * costoComb) *
       globs.HorasDelPaso;  //100 * AAcople +
   end
   else
     CostoDirectoDelPaso := 0;

 Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;
end;

procedure TGTer_combinado_horario.opt_leerSolucion_combinado(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;
  i: integer;
begin
 PDelPaso := 0;
 EnergiaDespachada := 0;
 NMaquinasDespachadasTG := 0;

 for i := 0 to NmaxTG - 1 do
 begin
  Xs_timer_[i] :=  s.xval(ivar_Xs_timer + i);

  if Xs_timer_[i] >= (5*0.9) then
     Xs_timer_[i] := 5
  else
  if Xs_timer_[i] <= AsumaCero then
     Xs_timer_[i] := 0;

  ATG_SC := trunc(s.xval(ivar_ATG_SC + 2*i) + 0.2);
  ATG_CC := trunc(s.xval(ivar_ATG_CC + 2*i) + 0.2);
  AAcople := trunc(s.xval(ivar_AAcople + i) + 0.2);

  NMaquinasDespachadasTG := NMaquinasDespachadasTG + ATG_SC + ATG_CC;

  if ATG_SC > 0 then
  begin
    BTG_SC := s.xval(ivar_BTG_SC + 2*i);
    PDelPaso := PDelPaso + BTG_SC + ATG_SC * PMinTG;
  end
  else
    BTG_SC := 0;

  if ATG_CC > 0 then
  begin
    BTG_CC := s.xval(ivar_BTG_CC + 2*i);
    PDelPaso := PDelPaso + BTG_CC + ATG_CC * PMinTG;
  end
  else
    BTG_CC := 0;

  if NMaquinasDespachadasTG > 0 then
  begin
    CostoDirectoDelPaso := CostoDirectoDelPaso + ( BTG_SC * cvTG + BTG_CC * cvTG + ATG_SC * c0TG +
    ATG_CC * c0TG + AAcople * costoComb) * globs.HorasDelPaso;  //100 * AAcople +
  end
  else
    CostoDirectoDelPaso := CostoDirectoDelPaso;

  end;

  PdespachTG := PDelPaso;

  ATV := trunc(s.xval(ivar_ATV) + 0.2);
  NMaquinasDespachadasTV := ATV;

  if ATV > 0 then
  begin
    BTV := s.xval(ivar_BTV);
    PDelPaso := PDelPaso + BTV + ATV * PMinTV;
  end
  else
  begin
    BTV := 0;
  end;
  PdespachTV := PDelPaso - PdespachTG;

  CostoDirectoDelPaso := CostoDirectoDelPaso + (BTV * cvTV + ATV * c0TV ) * globs.HorasDelPaso;


  EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
  P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

  Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;
end;
//funcion para no permitir que se combinen distintas TG en distintos momentos
//se pueden combinar de a una, o todas juntas, pero no puede estar combinada una
//y dsp entrar otra
procedure TGTer_combinado_horario.ChequearYActualizarTimers;
var
  aux: NReal;
  i: integer;
  flg_TimerActivo: boolean;
begin

 flg_TimerActivo := false;
 aux := 0;
 //busco el ultimo timer activo
 for i := NmaxTG - 1 downto 0 do
 begin
  if Xs_timer_[i] > 0 then
  begin
     aux := Xs_timer_[i];
     flg_TimerActivo := true;
     break;
  end;
 end;

 if flg_TimerActivo then
 begin
   //si se quieren prender en distintos momentos los mando a todos a purga
   for i := 0 to NmaxTG - 1 do
   begin
    if Xs_timer_[i] > 0 then
    begin
     if Xs_timer_[i] <> aux then
     begin
        Xs_timer_[i] := 1;
        aux := 1;
     end;
    end;
   end;
 end;

end;

procedure TGTer_combinado_horario.Sim_Cronica_Inicio;
var
  i: integer;
begin
  inherited Sim_Cronica_Inicio;
 // SetLength(X_timer_, 2);       //PRUEBA
 // SetLength(Xs_timer_, 2);
 for i := 0 to NmaxTG - 1 do
 begin
  X_timer_[i] := EstadoTimer;
  Xs_timer_[i] := X_timer_[i];
 end;

end;

function TGTer_combinado_horario.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (NMaquinasDisponiblesTG = 0) then
  begin
      Result := False;
  end
  else if ((NMaquinasDisponiblesTV = 0) or (NCalderasDisponibles = 0)) then
  begin
    getNombreVar_TV_indisponible(ivar, nombre);
  end
  else
  begin
    getNombreVar_combinado(ivar, nombre);
  end;

  {
  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (NMaquinasDisponiblesTV = 0) then
  begin
    getNombreVar_TV_indisponible(ivar, nombre);
  end
  else
  begin
    case purga_TV_[0] of
       0:
       begin
          getNombreVar_abierto_purga(ivar, nombre);
       end;
       1, 2:
       begin
          getNombreVar_combinado(ivar, nombre);
       end;
     end;
  end;
   }
end;

function TGTer_combinado_horario.getNombreVar_TV_indisponible(ivar: integer;
  var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + NmaxTG)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if (ivar < ivar_Xs_timer) and (((ivar - ivar_ATG_SC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr((ivar - ivar_ATG_SC) div (NmaxTG) + 1)
    else if (ivar < ivar_Xs_timer) and (((ivar - ivar_BTG_SC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr((ivar - ivar_BTG_SC) div (NmaxTG) + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

  {

  if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + 1)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr(ivar - ivar_BTG_SC + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;
   }
end;

function TGTer_combinado_horario.getNombreVar_abierto_purga(ivar: integer;
  var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + 1)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if ivar < ivar_ATG_CC then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr(ivar - ivar_BTG_SC + 1)
    else if ivar < ivar_AAcople then
      nombre := self.Nombre + '_A_TG_CC[MW]' + IntToStr(ivar - ivar_ATG_CC + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_AAcople' + IntToStr(ivar - ivar_AAcople + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreVar_combinado(ivar: integer;
  var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + NmaxTG)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if (ivar < ivar_ATG_CC) and (((ivar - ivar_ATG_SC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr((ivar - ivar_ATG_SC) div (NmaxTG) + 1)
    else if (ivar < ivar_ATG_CC) and (((ivar - ivar_BTG_SC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr((ivar - ivar_BTG_SC) div (NmaxTG) + 1)
    else if ivar < ivar_BTG_CC then
      nombre := self.Nombre + '_A_TG_CC[MW]' + IntToStr(ivar - ivar_ATG_CC + 1)
    else if (ivar < ivar_ATV) and (((ivar - ivar_ATG_CC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_A_TG_CC[MW]' + IntToStr(((ivar - ivar_ATG_CC) div (NmaxTG)) + 1)
    else if (ivar < ivar_ATV) and (((ivar - ivar_BTG_CC) mod (NmaxTG) = 0)) then
      nombre := self.Nombre + '_B_TG_CC' + IntToStr(((ivar - ivar_BTG_CC) div (NmaxTG)) + 1)
    else if ivar < ivar_BTV then
      nombre := self.Nombre + '_ATV[MW]' + IntToStr(ivar - ivar_ATV + 1)
    else if ivar < ivar_AAcople then
      nombre := self.Nombre + '_BTV' + IntToStr(ivar - ivar_BTV + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_AAcople' + IntToStr(ivar - ivar_AAcople + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreRes(ires: integer; var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if ((NMaquinasDisponiblesTV = 0) or (NCalderasDisponibles = 0)) then
  begin
    getNombreRes_TV_indisponible(ivar, nombre);
  end
  else
  begin
    getNombreRes_combinado(ires, nombre);
  end;

  {
  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (NMaquinasDisponiblesTV = 0) then
  begin
    getNombreVar_TV_indisponible(ivar, nombre);
  end
  else
  begin
    case purga_TV_[0] of
       0:
       begin
          getNombreRes_abierto_purga(ires, nombre);
       end;
       1, 2:
       begin
          getNombreRes_combinado(ires, nombre);
       end;
     end;
  end;
  }

end;

function TGTer_combinado_horario.getNombreRes_TV_indisponible(ires: integer;
  var nombre: string): boolean;
begin
  if (NMaquinasDisponiblesTG = 0) then
  begin
       Result := False;
  end
  else  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
  begin
      Result := False;
  end
  else
  begin
    if ires < jres_Acople_BTG_SC then
      nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
    else if ires < jres_Acople_TimerxBTG_SC then
      nombre := self.nombre + '_res-Acople_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
    else
      nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1);
    Result := True;
  end;


  {
  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
  begin
      Result := False;
  end
  else
  begin
    if ires < jres_Acople_BTG_SC then
      nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
    else if ires < jres_Acople_TimerxBTG_SC then
      nombre := self.nombre + '_res-extincion_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
    else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
    else if (pa.HayRestriccionEmaxPasoDeTiempo) then
    begin
    end;//por ahora nada
    Result := True;
  end;
  }
end;

function TGTer_combinado_horario.getNombreRes_abierto_purga(ires: integer;
  var nombre: string): boolean;
begin
 {
 jres_dinTimer := self.ires;
 jres_Acople_BTG_SC := jres_dinTimer + 1;
 jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;
 jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + 1;
 }

    if (NMaquinasDisponiblesTG = 0) then
    begin
         Result := False;
    end
    else  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
    begin
        Result := False;
    end
    else
    begin
      if ires < jres_Acople_BTG_SC then
        nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
      else if ires < jres_Acople_TimerxBTG_SC then
        nombre := self.nombre + '_res-extincion_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
      else if ires < jres_Acople_TimerxBTG_CC then
        nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
      else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_CC' + IntToStr(ires - jres_Acople_TimerxBTG_CC + 1)
      else if (pa.HayRestriccionEmaxPasoDeTiempo) then
      begin
      end;//por ahora nada
      Result := True;
    end;

end;

function TGTer_combinado_horario.getNombreRes_combinado(ires: integer;
  var nombre: string): boolean;
begin

    if (NMaquinasDisponiblesTG = 0) then
    begin
         Result := False;
    end
    else  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
    begin
        Result := False;
    end
    else
    begin
      if ires < jres_Acople_BTG_SC then
        nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
      else if ires < jres_Acople_BTG_CC then
        nombre := self.nombre + '_res-Acople_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
      else if ires < jres_Acople_BTV then
        nombre := self.nombre + '_res-Acople_BTG_CC' + IntToStr(ires - jres_Acople_BTG_CC + 1)
      else if ires < jres_Acople_TVxTG then
        nombre := self.nombre + '_res-Acople_BTV' + IntToStr(ires - jres_Acople_BTV + 1)
      else if ires < jres_Acople_TimerxBTG_SC then
        nombre := self.nombre + '_res-Acople_TVxTG' + IntToStr(ires - jres_Acople_TVxTG + 1)
      else if ires < jres_Acople_TimerxBTG_CC then
        nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
      else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_CC' + IntToStr(ires - jres_Acople_TimerxBTG_CC + 1)
      else if (pa.HayRestriccionEmaxPasoDeTiempo) then
      begin
      end;//por ahora nada
      Result := True;
    end;

end;

procedure TGTer_combinado_horario.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableNI('NDespachadasTGs', '-', NMaquinasDespachadasTG, True);
  PublicarVariableNI('NDespachadasTVs', '-', NMaquinasDespachadasTV, True);
  {$IFDEF DECLARAR_VARIABLES_SIMRES_DEF}
  declararVarsPSimResPorDefectoIntercalandoPostes(['P', 'Costo', 'NMaqsDespachadas'],
    globs.NPostes);
  {$ENDIF}
  PublicarVariableNR('c0TG', '[USD/h]', 6, 1, c0TG, False);
  PublicarVariableNI('NMaqsDisponiblesTG', '-', NMaquinasDisponiblesTG, False);
  PublicarVariableNI('NMaqsDisponiblesTV', '-', NMaquinasDisponiblesTV, False);
  PublicarVariableNR('PMaxDisponible_Central', '[MW]', 6, 1,
    PMaxDisponible_Central, False);
  PublicarVariableNR('PMediaDespachada', '[MW]', 6, 1, potMedia_despachada, False);
  PublicarVariableVR('Timer', '[h]', 6, 1, X_timer_, False, True);
  PublicarVariableNR('PTG', '[MW]', 6, 1, PdespachTG, False);
  PublicarVariableNR('PTV', '[MW]', 6, 1, PdespachTV, False);

end;

destructor TGTer_combinado_horario.destroy;
begin

  setlength( X_timer_, 0 );
  setlength( Xs_timer_, 0 );
  setlength( purga_TV_, 0 );

  inherited destroy;
end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TGTer_combinado_horario.ClassName, TGTer_combinado_horario);
  registrarClaseDeCosa(TFichaGTer_combinado_horario.ClassName, TFichaGTer_combinado_horario);
end;

procedure AlFinal;
begin
end;

end.
